@@ -21,28 +21,28 @@ import java.util.Collections;
 import java.util.List;
 
 import org.locationtech.jts.geom.Geometry;
+import org.locationtech.jts.geom.GeometryCollection;
 import org.locationtech.jts.geom.GeometryFactory;
+import org.locationtech.jts.geom.PrecisionModel;
 import org.locationtech.jts.index.SpatialIndex;
 import org.locationtech.jts.index.strtree.STRtree;
 import org.locationtech.jts.util.Stopwatch;
 import org.locationtech.jtstest.geomfunction.GeometryFunction;
 import org.locationtech.jtstest.geomfunction.GeometryFunctionRegistry;
+import org.locationtech.jtstest.geomfunction.FilterGeometryFunction;
 import org.locationtech.jtstest.testbuilder.ui.SwingUtil;
-import org.locationtech.jtstest.util.io.IOUtil;
 import org.locationtech.jtstest.util.io.MultiFormatBufferedReader;
+import org.locationtech.jtstest.util.io.MultiFormatFileReader;
 import org.locationtech.jtstest.util.io.MultiFormatReader;
 
 /**
- * Runs an operation according to suppliec parameters.
+ * Runs an operation according to supplied parameters.
  * 
  * @author Martin Davis
  *
  */
 public class JTSOpRunner {
 
-  // TODO: add option -ab to read both geoms from a file
-  // TODO: allow -a stdin  to indicate reading from stdin.  
-  
   public static final String ERR_FILE_NOT_FOUND = "File not found";
   public static final String ERR_INPUT = "Unable to read input";
   public static final String ERR_PARSE_GEOM = "Unable to parse geometry";
@@ -71,29 +71,57 @@ public class JTSOpRunner {
   private String symGeom2 = SYM_B;
 
   private IndexedGeometry geomIndexB;
-  private Geometry geomA;
-  private Geometry geomB;
+  private List<Geometry> geomA;
+  private List<Geometry> geomB;
   private OpParams param;
   private String hdrSave;
   private long totalTime;
   private int opCount = 0;
+  private boolean isTime;
   
   static class OpParams {
-    String operation;
+    static final int OFFSET_DEFAULT = 0;
+    static final int LIMIT_DEFAULT = -1;
+    
     public String fileA;
     String geomA;
+    public int limitA = LIMIT_DEFAULT;
+    public int offsetA = OFFSET_DEFAULT;
+    
     public String fileB;
     public String geomB;
+    public int limitB = LIMIT_DEFAULT;
+    public int offsetB = OFFSET_DEFAULT;
+    
     public boolean isGeomAB = false;
-    //public String[] arg1;
+    public boolean isCollect = false;
     String format = null;
     public Integer repeat;
     public boolean eachA = false;
     public boolean eachB = false;
     public boolean eachAA = false;
-    public String[] argList;
     public boolean validate = false;
     public boolean isIndexed = false;
+    public boolean isExplode = false;
+    public int srid;
+    
+    public boolean isFilter = false;
+    public int filterOp;
+    public double filterVal = 0;
+    
+    String operation;
+    public String[] argList;
+    
+    /**
+     * Tests whether an input geometry has been supplied.
+     * 
+     * @param file
+     * @param geom
+     * @return true if an input geometry is present
+     */
+    static boolean isGeometryInput(String file, String geom) {
+      return file != null || geom != null;
+    }
   }
 
   public JTSOpRunner() {
@@ -107,7 +135,9 @@ public class JTSOpRunner {
   public void setVerbose(boolean isVerbose) {
     this.isVerbose = isVerbose;
   }
-  
+  public void setTime(boolean isTime) {
+    this.isTime = isTime;
+  }
   public void captureOutput() {
     out = new CommandOutput(true);
     geomOut = new GeometryOutput(out);
@@ -131,25 +161,26 @@ public class JTSOpRunner {
   void execute(OpParams param) {
     this.param = param;
     
+    geomFactory = createGeometryFactory(param.srid);
     geomA = null;
     geomB = null;
 
     loadGeometry();
     if (geomA != null) {
-      printGeometrySummary("A", geomA, param.fileA);
+      printGeometrySummary("A", geomA, fileInfo(param.fileA, param.limitA, param.offsetA) );
     }
     if (geomB != null) {
-      printGeometrySummary("B", geomB, param.fileB);
+      printGeometrySummary("B", geomB, fileInfo(param.fileB, param.limitB, param.offsetB) );
     }
     
-    //--- If -each aa specified, use A for B
-    if (param.eachAA) {
+    //--- If -ab aa specified, use A for B
+    if (param.isGeomAB) {
       geomB = geomA;
       symGeom2  = SYM_A;
     }
 
-    // index B if requested
-    if (param.eachB) {
+    // index B if present and requested
+    if (geomB != null) {
       geomIndexB = new IndexedGeometry(geomB, param.isIndexed);
     }
     
@@ -158,31 +189,74 @@ public class JTSOpRunner {
     }
     else {
       // no op specified, so just output A (allows format conversion)
-      printResult(geomA, param.format);
+      outputList(geomA, param.format);
     }
   }
 
+  private GeometryFactory createGeometryFactory(int srid) {
+    if (srid > 0) {
+      return new GeometryFactory(new PrecisionModel(), srid);
+    }
+    return new GeometryFactory();
+  }
+
   private void loadGeometry() {
-    if (param.isGeomAB) {
-      loadGeometryAB();
+    geomA = readGeometry("A", param.fileA, param.geomA, param.limitA, param.offsetA);
+    geomB = readGeometry("B", param.fileB, param.geomB, param.limitB, param.offsetB);
+    
+    if (param.eachA) {
+      geomA = explode(geomA);
     }
-    else {
-      geomA = readGeometry("A", param.fileA, param.geomA);
-      geomB = readGeometry("B", param.fileB, param.geomB);
+    if (param.eachB) {
+      geomB = explode(geomB);
+    }
+    if (param.isCollect) {
+      geomA = collect(geomA, geomFactory);
     }
   }
 
+  private static List<Geometry> collect(List<Geometry> geoms, GeometryFactory factory) {
+    GeometryCollection geomColl = factory.createGeometryCollection(
+        GeometryFactory.toGeometryArray(geoms));
+    return toList(geomColl);
+  }
+  
+  private static List<Geometry> explode(List<Geometry> geoms) {
+    if (geoms == null) return null;
+    List<Geometry> geomsEx = new ArrayList<Geometry>();
+    for (Geometry geom : geoms) {
+      explode(geom, geomsEx);
+    }
+    return geomsEx;
+  }
+
+  private static void explode(Geometry geom, List<Geometry> geomsEx) {
+    for (int i = 0; i < geom.getNumGeometries(); i++) {
+      geomsEx.add(geom.getGeometryN(i));
+    }
+  }
+
+  private static List<Geometry> toList(Geometry geometry) {
+    List<Geometry> geoms = new ArrayList<Geometry>();
+    geoms.add(geometry);
+    return geoms;
+  }
+  
   private void loadGeometryAB() {
-    Geometry geomAB = readGeometry("AB", param.fileA, param.geomA);
-    if (geomAB.getNumGeometries() < 2) {
+    List<Geometry> geomAB = readGeometry("AB", param.fileA, param.geomA, OpParams.LIMIT_DEFAULT, OpParams.OFFSET_DEFAULT);
+    if (geomAB.size() < 2) {
       throw new CommandError(ERR_REQUIRED_B);
     }
-    geomA = geomAB.getGeometryN(0);
-    geomB = geomAB.getGeometryN(1);
+    geomA = toList(geomAB.get(0));
+    geomB = toList(geomAB.get(1));
   }
 
   private void executeFunction() {
-    GeometryFunction func = getFunction(param.operation);
+    GeometryFunction baseFun = getFunction(param.operation);
+    GeometryFunction func = baseFun;
+    if (param.isFilter) {
+      func = new FilterGeometryFunction(func, param.filterOp, param.filterVal);
+    }
     
     if (func == null) {
       throw new CommandError(ERR_FUNCTION_NOT_FOUND, param.operation);
@@ -191,69 +265,55 @@ public class JTSOpRunner {
     checkFunctionArgs(func, geomB, argList);
 
     FunctionInvoker fun = new FunctionInvoker(func, argList);
-    executeFunctionSpreadA(fun);
+    executeFunctionOverA(fun);
     
-    printlnInfo("\nOperations: " + opCount
-        + "  Total Time: " + Stopwatch.getTimeString( totalTime ));
+    if (isVerbose || isTime) {
+      out.println("\nOperation " + func.getCategory() + "." + func.getName() + ": " + opCount
+        + " invocations - Total Time: " + Stopwatch.getTimeString( totalTime ));
+    }
   }
   
-  private void executeFunctionSpreadA(FunctionInvoker fun) {
+  private void executeFunctionOverA(FunctionInvoker fun) {
     int numGeom = 1;
     if (geomA != null) {
-      numGeom = geomA.getNumGeometries(); 
+      numGeom = geomA.size(); 
     }
-    String header = "\n";
-    boolean isSpread = param.eachA && numGeom > 1;
-    if (! isSpread) {
-      executeFunctionSpreadB(geomA, fun, header);
-      return;
-    }
-    
-    // spread over A
+    String header = "";
     for (int i = 0; i < numGeom; i++) {
-      Geometry comp = geomA.getGeometryN(i);
-      String hdr =  "\n" + GeometryOutput.writeGeometrySummary(SYM_A + "[" + i + "]", comp);
-      executeFunctionSpreadB(comp, fun, hdr);
+      Geometry comp = geomA == null ? null : geomA.get(i);
+      String hdr =  GeometryOutput.writeGeometrySummary(SYM_A + "[" + i + "]", comp);
+      if (geomB == null) {
+        executeFunction(comp, fun, hdr);
+      }
+      else {
+        executeFunctionOverB(comp, fun, hdr);
+      }
     }
   }
-  
-  private void executeFunctionSpreadB(Geometry geomA, FunctionInvoker fun, String header) {
-    int numGeom = 1;
-    if (fun.isBinaryGeom()) {
-      numGeom = geomB.getNumGeometries();
-    }
-    boolean isSpread = geomB != null 
-        && (param.eachB || param.eachAA )
-        && numGeom > 1;
-        
-    if (! isSpread) {
-      fun.setB(geomB);
-      executeFunctionArgs(geomA, fun, header);
-      return;
-    }
-    
+
+  private void executeFunctionOverB(Geometry geomA, FunctionInvoker fun, String header) {
     // spread over B
     List<Integer> targetB = geomIndexB.query(geomA);
     for (int index : targetB) {
-      Geometry comp = geomB.getGeometryN(index);
-      String hdr = header + "\n" + GeometryOutput.writeGeometrySummary(symGeom2 + "[" + index + "]", comp);
-      fun.setB(comp);
-      executeFunctionArgs(geomA, fun, hdr);
+      Geometry gb = geomB.get(index);
+      String hdr = header + ", " + GeometryOutput.writeGeometrySummary(symGeom2 + "[" + index + "]", gb);
+      fun.setB(gb);
+      executeFunction(geomA, fun, hdr);
     }
   }
   
-  private void executeFunctionArgs(Geometry geomA, FunctionInvoker fun, String hdr) {
+  private void executeFunction(Geometry geomA, FunctionInvoker fun, String hdr) {
     // Set saved hdr to blank in case verbose is on
     hdrSave = "";
-    printlnInfo(hdr);
+    //printlnInfo(hdr);
     for (int i = 0; i < fun.getNumInvocations(); i++) {
       Object funArgs[] = fun.getArgs(i);
       GeometryFunction func = fun.getFunction();
       String arg = fun.getValue(i);
       
-      String opDesc = "-- " + opSummary(func.getName(), arg) + "  ------------------------";
+      String opDesc = "[" + (opCount+1) + "] -- " + opSummary(func, arg) + " : ";
       if (isVerbose) {
-        out.println(opDesc);
+        out.println(opDesc + hdr);
       }
       else {
         hdrSave = hdr + "\n" + opDesc;
@@ -296,22 +356,26 @@ public class JTSOpRunner {
     opCount++;
     
     if (result instanceof Geometry) {
-      printGeometrySummary("Result", (Geometry) result, null);
+      printGeometrySummary("Result", (Geometry) result);
     }
     if (param.validate) {
       validate(result);
     }
-    printResult(result, param.format);
+    outputResult(result, param.isExplode, param.format);
     return result;
   }
 
   private String errorMsg(Throwable ex) {
     String msg = "ERROR excuting function: " + ex.getMessage() + "\n";
-    msg += toStrackString(ex);
+    msg += toStackString(ex);
+    if (ex.getCause() != null) {
+      msg += "Caused by:\n";
+      msg += toStackString(ex.getCause());
+    }
     return msg;
   }
 
-  private String toStrackString(Throwable ex) {
+  private String toStackString(Throwable ex) {
     StringWriter sw = new StringWriter();
     PrintWriter pw = new PrintWriter(sw);
     ex.printStackTrace(pw);
@@ -334,31 +398,32 @@ public class JTSOpRunner {
       logError("Result is invalid");
     }
   }
-
+  
   /**
    * Reads a geometry from a literal or a filename.
    * If neither are provided this geometry is not present.
    * 
    * @param geomLabel label for geometry being read
-   * @param filename the filename to read from, if present
-   * @param geom the geometry literal, if present
+   * @param filename the filename to read from, if present, or <code>null</code>
+   * @param geom the geometry literal, if present, or <code>null</code>
    * @param geomA2 
    * @return the geometry read, or null
    * @throws Exception
    */
-  private Geometry readGeometry(String geomLabel, String filename, String geom) {
+  private List<Geometry> readGeometry(String geomLabel, String filename, String geomStr, int limit, int offset) {
     String geomDesc = " " + geomLabel + " ";
-    if (geom != null) {
+    if (geomStr != null) {
       // read a literal from the argument
       MultiFormatReader rdr = new MultiFormatReader(geomFactory);
       try {
-        return rdr.read(geom);
+        Geometry g = rdr.read(geomStr);
+        return toList(g);
       } 
       catch (org.locationtech.jts.io.ParseException ex) {
         throw new CommandError(ERR_PARSE_GEOM + geomDesc + " - " + ex.getMessage());
       }
       catch (Exception e) {
-        throw new CommandError(ERR_PARSE_GEOM + geomDesc, limitLength(geom, 50));
+        throw new CommandError(ERR_PARSE_GEOM + geomDesc, limitLength(geomStr, 50));
       }
     }
     // no parameter supplied
@@ -366,11 +431,11 @@ public class JTSOpRunner {
     
     // must be a filename
     if (filename.equalsIgnoreCase(CommandOptions.STDIN)){
-      return readStdin();     
+      return readStdin(limit, offset);     
     }
     
     try {
-      return IOUtil.readFile(filename ,geomFactory );
+      return MultiFormatFileReader.read(filename, limit, offset, geomFactory );
     }
     catch (FileNotFoundException ex) {
       throw new CommandError(ERR_FILE_NOT_FOUND, filename);
@@ -379,10 +444,9 @@ public class JTSOpRunner {
     }
   }
 
-  private Geometry readStdin() {
+  private List<Geometry> readStdin(int limit, int offset) {
     try {
-      MultiFormatBufferedReader rdr = new MultiFormatBufferedReader();
-      return rdr.read(new InputStreamReader(stdIn));
+      return MultiFormatBufferedReader.read(new InputStreamReader(stdIn), limit, offset, geomFactory);
     }
     catch (org.locationtech.jts.io.ParseException ex) {
       throw new CommandError(ERR_PARSE_GEOM + " - " + ex.getMessage());
@@ -397,36 +461,58 @@ public class JTSOpRunner {
     return s.substring(0, n) + "...";
   }
   
-  private static String opSummary(String funcName, String arg) {
+  private static String opSummary(GeometryFunction func, String arg) {
     StringBuilder sb = new StringBuilder();
-    sb.append("Op: " + funcName );
+    sb.append("Op: " + func.getCategory() + "." + func.getName() );
     if (arg != null) {
       sb.append(" " + arg);
     }
     return sb.toString();
   }
 
-  private void printResult(Object result, String outputFormat) {
+  private void outputResult(Object result, boolean isExplode, String outputFormat) {
     if (result == null) return;
     if (outputFormat == null) return;
     
-    if (result instanceof Geometry) {
-      if (captureGeometry) {
-        resultGeoms.add((Geometry) result);
+    if (! (result instanceof Geometry)) {
+      out.println(result);
+      return;
+    }
+    Geometry geom = (Geometry) result;
+    if (isExplode && geom instanceof GeometryCollection) {
+      for (int i = 0; i < geom.getNumGeometries(); i++) {
+        printGeometry(geom.getGeometryN(i), param.srid, outputFormat);
       }
-      geomOut.printGeometry((Geometry) result, outputFormat);
     }
     else {
-      out.println(result);
+      printGeometry(geom, param.srid, outputFormat);
     }
   }
+  private void outputList(List<Geometry> geoms, String outputFormat) {
+    if (geoms == null) return;
+    if (outputFormat == null) return;
+
+    for (Geometry geom : geoms) {
+      outputResult(geom, param.isExplode, outputFormat);
+    }
+  }
+  
+  private void printGeometry(Geometry geom, int srid, String outputFormat) {
+    if (geom == null) return;
+    if (outputFormat == null) return;
+    
+    if (captureGeometry) {
+      resultGeoms.add((Geometry) geom);
+    }
+    geomOut.printGeometry((Geometry) geom, srid, outputFormat);
+  }
   
   private void printlnInfo(String s) {
     if (! isVerbose) return;
     out.println(s);
   }
   
-  private void printGeometrySummary(String label, Geometry geom, String source) {
+  private void printGeometrySummary(String label, List<Geometry> geom, String source) {
     // short-circuit to avoid cost
     if (! isVerbose) return;
     
@@ -435,33 +521,49 @@ public class JTSOpRunner {
     printlnInfo( GeometryOutput.writeGeometrySummary(label, geom) + srcname);
   }
   
-  private void checkFunctionArgs(GeometryFunction func, Geometry geomB, String[] argList) {
-    Class[] paramTypes = func.getParameterTypes();
+  private void printGeometrySummary(String label, Geometry geom) {
+    // short-circuit to avoid cost
+    if (! isVerbose) return;
+    printlnInfo( GeometryOutput.writeGeometrySummary(label, geom));
+  }
+  
+  private static String fileInfo(String filename, int limit, int offset) {
+    if (filename == null) return null;
+    String info = filename;
+    if (limit > OpParams.LIMIT_DEFAULT) info += " LIMIT " + limit;
+    if (offset > OpParams.OFFSET_DEFAULT) info += " OFFSET " + offset;
+    return info;
+  }
+  
+  private void checkFunctionArgs(GeometryFunction func, List<Geometry> geomB, String[] argList) {
+    Class<?>[] paramTypes = func.getParameterTypes();
     int nParam = paramTypes.length;
     
+    /*
+    // disable this check for now, since it does not handle functions where B is optional
     if (func.isBinary() && geomB == null)
       throw new CommandError(ERR_REQUIRED_B);
-    /**
-    // MD not sure whether to check this?
-    if (! func.isBinary() && geomB != null)
-      throw new CommandError(ERR_REQUIRED_B);
-      */
+     */
     
     /*
      * check count of supplied args.
      * Assumes B has been checked.
      */
     int argCount = 0;
-    if (func.isBinary() && geomB != null) argCount++;
+    if (func.isBinary()
+      // disable B check for now
+      //  && geomB != null
+        ) {
+      argCount++;
+    }
     if (argList != null) argCount++;
     if (nParam != argCount) {
       throw new CommandError(ERR_WRONG_ARG_COUNT, func.getName());
     }
   }
   
-
-  
   private GeometryFunction getFunction(String operation) {
+    // default category is Geometry
     String category = "Geometry";
     String name = operation;
     String[] opCatName = operation.split("\\.");
@@ -471,6 +573,11 @@ public class JTSOpRunner {
     }
     return funcRegistry.find(category, name);
   }
+
+  public static boolean isCustomSRID(int srid) {
+    return srid > 0;
+  }
+
 }
 
 class FunctionInvoker {
@@ -512,7 +619,7 @@ class FunctionInvoker {
   }
   
   private Object[] createFunctionArgs(GeometryFunction func, Geometry geomB, String arg1) {
-    Class[] paramTypes = func.getParameterTypes();
+    Class<?>[] paramTypes = func.getParameterTypes();
     Object[] paramVal = new Object[paramTypes.length];
     
     int iparam = 0;
@@ -532,28 +639,28 @@ class IndexedGeometry
   private SpatialIndex index = null;
   private List<Integer> allIndexes = null;
   
-  public IndexedGeometry(Geometry geom, boolean isIndexed)
+  public IndexedGeometry(List<Geometry> geoms, boolean isIndexed)
   {
     if (isIndexed) {
-      initIndex(geom);
+      initIndex(geoms);
     }
     else {
-      initList(geom);
+      initList(geoms);
     }
   }
   
-  private void initList(Geometry geom) {
+  private void initList(List<Geometry> geoms) {
     allIndexes = new ArrayList<Integer>();
-    for (int i = 0; i < geom.getNumGeometries(); i++) {
+    for (int i = 0; i < geoms.size(); i++) {
       allIndexes.add(i);
     }
   }
 
-  private void initIndex(Geometry geom)
+  private void initIndex(List<Geometry> geoms)
   {
     index = new STRtree();
-    for (int i = 0; i < geom.getNumGeometries(); i++) {
-      Geometry comp = geom.getGeometryN(i);
+    for (int i = 0; i < geoms.size(); i++) {
+      Geometry comp = geoms.get(i);
       index.insert(comp.getEnvelopeInternal(), new Integer(i));
     }
   }
