@@ -13,6 +13,8 @@ package org.locationtech.jtstest.function;
 
 import org.locationtech.jts.geom.Geometry;
 import org.locationtech.jts.operation.overlay.OverlayOp;
+import org.locationtech.jts.operation.union.UnaryUnionOp;
+import org.locationtech.jts.operation.union.UnionStrategy;
 
 public class OverlayNoSnapFunctions {
 	public static Geometry intersection(Geometry a, Geometry b)		{		return OverlayOp.overlayOp(a, b, OverlayOp.INTERSECTION);	}
@@ -21,4 +23,21 @@ public class OverlayNoSnapFunctions {
 	public static Geometry difference(Geometry a, Geometry b)			{		return OverlayOp.overlayOp(a, b, OverlayOp.DIFFERENCE);	}
 	public static Geometry differenceBA(Geometry a, Geometry b)		{		return OverlayOp.overlayOp(b, a, OverlayOp.DIFFERENCE);	}
 
+	 public static Geometry unaryUnion(Geometry a) {
+	    UnionStrategy unionSRFun = new UnionStrategy() {
+
+	      public Geometry union(Geometry g0, Geometry g1) {
+	         return OverlayOp.overlayOp(g0, g1, OverlayOp.UNION );
+	      }
+
+	      @Override
+	      public boolean isFloatingPrecision() {
+	        return true;
+	      }
+	      
+	    };
+	    UnaryUnionOp op = new UnaryUnionOp(a);
+	    op.setUnionFunction(unionSRFun);
+	    return op.union();
+	  }
 }
