@@ -13,28 +13,23 @@ package org.locationtech.jtstest.testbuilder;
 
 import java.awt.AWTEvent;
 import java.awt.BorderLayout;
-import java.awt.Color;
 import java.awt.Component;
 import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.GridBagLayout;
 import java.awt.GridLayout;
-import java.awt.Toolkit;
 import java.awt.event.ActionEvent;
 import java.awt.event.WindowEvent;
 import java.io.File;
 import java.util.Iterator;
 import java.util.List;
 
-import javax.swing.BorderFactory;
-import javax.swing.ImageIcon;
 import javax.swing.JFileChooser;
 import javax.swing.JFrame;
 import javax.swing.JOptionPane;
 import javax.swing.JPanel;
 import javax.swing.JSplitPane;
 import javax.swing.JTabbedPane;
-import javax.swing.border.BevelBorder;
 import javax.swing.border.Border;
 import javax.swing.border.EmptyBorder;
 import javax.swing.event.ChangeEvent;
@@ -42,36 +37,15 @@ import javax.swing.event.ChangeListener;
 
 import org.locationtech.jts.geom.Coordinate;
 import org.locationtech.jts.geom.Geometry;
-import org.locationtech.jts.geom.util.LinearComponentExtracter;
-import org.locationtech.jts.io.ParseException;
 import org.locationtech.jts.util.Assert;
-import org.locationtech.jtstest.clean.CleanDuplicatePoints;
-import org.locationtech.jtstest.testbuilder.controller.ResultController;
-import org.locationtech.jtstest.testbuilder.event.SpatialFunctionPanelEvent;
-import org.locationtech.jtstest.testbuilder.event.SpatialFunctionPanelListener;
-import org.locationtech.jtstest.testbuilder.io.HtmlWriter;
-import org.locationtech.jtstest.testbuilder.io.JavaTestWriter;
 import org.locationtech.jtstest.testbuilder.io.XMLTestWriter;
 import org.locationtech.jtstest.testbuilder.model.DisplayParameters;
 import org.locationtech.jtstest.testbuilder.model.GeometryEvent;
 import org.locationtech.jtstest.testbuilder.model.TestBuilderModel;
 import org.locationtech.jtstest.testbuilder.model.TestCaseEdit;
-import org.locationtech.jtstest.testbuilder.ui.ImageUtil;
 import org.locationtech.jtstest.testbuilder.ui.SwingUtil;
 import org.locationtech.jtstest.testbuilder.ui.dnd.FileDrop;
-import org.locationtech.jtstest.testbuilder.ui.tools.DeleteVertexTool;
-import org.locationtech.jtstest.testbuilder.ui.tools.EditVertexTool;
-import org.locationtech.jtstest.testbuilder.ui.tools.ExtractComponentTool;
-import org.locationtech.jtstest.testbuilder.ui.tools.InfoTool;
-import org.locationtech.jtstest.testbuilder.ui.tools.LineStringTool;
-import org.locationtech.jtstest.testbuilder.ui.tools.PanTool;
-import org.locationtech.jtstest.testbuilder.ui.tools.PointTool;
 import org.locationtech.jtstest.testbuilder.ui.tools.RectangleTool;
-import org.locationtech.jtstest.testbuilder.ui.tools.StreamPolygonTool;
-import org.locationtech.jtstest.testbuilder.ui.tools.Tool;
-import org.locationtech.jtstest.testbuilder.ui.tools.ZoomTool;
-import org.locationtech.jtstest.testrunner.GuiUtil;
-import org.locationtech.jtstest.util.FileUtil;
 import org.locationtech.jtstest.util.StringUtil;
 
 
@@ -84,7 +58,10 @@ public class JTSTestBuilderFrame extends JFrame
 {
     
   private static JTSTestBuilderFrame singleton = null;
-  private ResultController resultController = new ResultController(this);
+  static boolean isShowingIndicators = true;
+  
+  TestBuilderModel tbModel;
+
   private JTSTestBuilderMenuBar tbMenuBar = new JTSTestBuilderMenuBar(this);
   private JTSTestBuilderToolBar tbToolBar = new JTSTestBuilderToolBar(this);
   //---------------------------------------------
@@ -111,25 +88,12 @@ public class JTSTestBuilderFrame extends JFrame
   ResultValuePanel resultValuePanel = new ResultValuePanel();
   StatsPanel statsPanel = new StatsPanel();
   InfoPanel logPanel = new InfoPanel();
-  private ZoomTool zoomTool;
-  private final ImageIcon appIcon = new ImageIcon(this.getClass().getResource("app-icon.gif"));
 
   private JFileChooser fileChooser = new JFileChooser();
   private JFileChooser pngFileChooser;
   private JFileChooser fileAndDirectoryChooser = new JFileChooser();
   private JFileChooser directoryChooser = new JFileChooser();
   
-  TestBuilderModel tbModel;
-  
-  private TestCaseTextDialog testCaseTextDlg = new TestCaseTextDialog(this,
-      "", true);
-  private GeometryInspectorDialog geomInspectorDlg = new GeometryInspectorDialog(this);
-  /*
-  private LoadTestCasesDialog loadTestCasesDlg = new LoadTestCasesDialog(this,
-      "Load Test Cases", true);
-*/
-  
-  
   /**
    *  Construct the frame
    */
@@ -138,36 +102,22 @@ public class JTSTestBuilderFrame extends JFrame
       Assert.isTrue(singleton == null);
       singleton = this;
       enableEvents(AWTEvent.WINDOW_EVENT_MASK);
-      setIconImage(appIcon.getImage());
+      setIconImage(AppIcons.APP.getImage());
       jbInit();
-
-      testCasePanel.spatialFunctionPanel.addSpatialFunctionPanelListener(
-          new SpatialFunctionPanelListener() {
-            public void functionExecuted(SpatialFunctionPanelEvent e) {
-            	resultController.spatialFunctionPanel_functionExecuted(e);
-            }
-          });
-      testCasePanel.scalarFunctionPanel.addSpatialFunctionPanelListener(
-          new SpatialFunctionPanelListener() {
-            public void functionExecuted(SpatialFunctionPanelEvent e) {
-            	resultController.executeScalarFunction();
-            }
-          });
       testCasePanel.cbRevealTopo.addActionListener(
           new java.awt.event.ActionListener() {
             public void actionPerformed(ActionEvent e) {
-              revealTopo_actionPerformed();
+              displayRevealTopo();
             }
           });
       //testCasePanel.editCtlPanel.stretchDist
       testCasePanel.spStretchDist
       .addChangeListener(new javax.swing.event.ChangeListener() {
         public void stateChanged(javax.swing.event.ChangeEvent e) {
-          revealTopo_actionPerformed();
+          displayRevealTopo();
         }
       });
 
-      zoomTool = new ZoomTool(2, AppCursors.ZOOM);
       showGeomsTab();
       initFileDrop(testCasePanel);
       testCasePanel.getGeometryEditPanel().setCurrentTool(RectangleTool.getInstance());
@@ -203,12 +153,31 @@ public class JTSTestBuilderFrame extends JFrame
     }
     return singleton;
   }
-
+  /**
+   * Tests if the TestBuilder is running.
+   * Useful to allow functions to decide whether to show indicators 
+   * (if functions are running under JtsOpCmd, they should not show indicators
+   * since that seriously impacts performance).
+   * 
+   * @return true if there is a TestBuilder instance running
+   */
+  public static boolean isRunning() {
+    return singleton != null;
+  }
+  public static boolean isShowingIndicators() {
+    return isRunning() && isShowingIndicators;
+  }
+  
   public static GeometryEditPanel getGeometryEditPanel()
   {
     return instance().getTestCasePanel().getGeometryEditPanel();
   }
-  
+
+  public static SpatialFunctionPanel getSpatialFunctionPanel()
+  {
+    return instance().getTestCasePanel().spatialFunctionPanel;
+  }
+
   public TestBuilderModel getModel()
   {
     return tbModel;
@@ -283,20 +252,6 @@ public class JTSTestBuilderFrame extends JFrame
     System.exit(0);
   }
 
-  /**
-   *  Help | About action performed
-   */
-  public void jMenuHelpAbout_actionPerformed(ActionEvent e) {
-    JTSTestBuilder_AboutBox dlg = new JTSTestBuilder_AboutBox(this);
-    java.awt.Dimension dlgSize = dlg.getPreferredSize();
-    java.awt.Dimension frmSize = getSize();
-    java.awt.Point loc = getLocation();
-    dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height
-         - dlgSize.height) / 2 + loc.y);
-    dlg.setModal(true);
-    dlg.setVisible(true);
-  }
-
   public void showTab(String name)
   {
     inputTabbedPane.setSelectedIndex(inputTabbedPane.indexOfTab(name));
@@ -344,8 +299,7 @@ public class JTSTestBuilderFrame extends JFrame
   }
 
   void model_geometryChanged(GeometryEvent e) {
-    //testCasePanel.relatePanel.clearResults();
-    JTSTestBuilder.controller().geometryViewChanged();
+    JTSTestBuilder.controller().geometryChanged();
     updateWktPanel();
   }
 
@@ -374,40 +328,21 @@ public class JTSTestBuilderFrame extends JFrame
     Object currResult = tbModel.getResult();
     if (! (currResult instanceof Geometry))
       return;
-    inspectGeometry((Geometry) currResult, 0, "R");
-  }
-  
-  void menuViewText_actionPerformed(ActionEvent e) {
-    testCaseTextDlg.setTestCase(currentCase());
-    testCaseTextDlg.setVisible(true);
+    inspectGeometry((Geometry) currResult, 0, "R", false);
   }
 
-  public void actionInspectGeometry() {
+  public void inspectGeometry() {
     int geomIndex = tbModel.getGeometryEditModel().getGeomIndex();
     String tag = geomIndex == 0 ? AppStrings.GEOM_LABEL_A : AppStrings.GEOM_LABEL_B;
     Geometry geometry = currentCase().getGeometry(geomIndex);
-    inspectGeometry(geometry, geomIndex, tag);
-    /*
-    geomInspectorDlg.setGeometry(
-        geomIndex == 0 ? AppStrings.GEOM_LABEL_A : AppStrings.GEOM_LABEL_B,
-        tbModel.getCurrentTestCaseEdit().getGeometry(geomIndex));
-        */
-    //geomInspectorDlg.setVisible(true);
+    inspectGeometry(geometry, geomIndex, tag, true);
   }
 
-  public void inspectGeometry(Geometry geometry, int geomIndex, String tag) {
-    inspectPanel.setGeometry( tag, geometry, geomIndex);
+  private void inspectGeometry(Geometry geometry, int geomIndex, String tag, boolean isEditable) {
+    inspectPanel.setGeometry( tag, geometry, geomIndex, isEditable);
     showTab(AppStrings.TAB_LABEL_INSPECT);
   }
 
-  public void actionInspectGeometryDialog() {
-    int geomIndex = tbModel.getGeometryEditModel().getGeomIndex();
-    String tag = geomIndex == 0 ? AppStrings.GEOM_LABEL_A : AppStrings.GEOM_LABEL_B;
-    Geometry geometry = currentCase().getGeometry(geomIndex);
-    geomInspectorDlg.setGeometry(tag, geometry);
-    geomInspectorDlg.setVisible(true);
-  }
-
   void menuLoadXmlTestFile_actionPerformed(ActionEvent e) {
     try {
       fileChooser.removeChoosableFileFilter(SwingUtil.JAVA_FILE_FILTER);
@@ -427,115 +362,10 @@ public class JTSTestBuilderFrame extends JFrame
     }
   }
 
-  void menuSaveAsXml_actionPerformed(ActionEvent e) {
-    try {
-      fileChooser.removeChoosableFileFilter(SwingUtil.JAVA_FILE_FILTER);
-      fileChooser.addChoosableFileFilter(SwingUtil.XML_FILE_FILTER);
-      fileChooser.setDialogTitle("Save XML Test File");
-      if (JFileChooser.APPROVE_OPTION == fileChooser.showSaveDialog(this)) {
-        File file = fileChooser.getSelectedFile();
-        if (! SwingUtil.confirmOverwrite(this, file)) return;
-        FileUtil.setContents(fileChooser.getSelectedFile().getPath(), 
-        		XMLTestWriter.getRunXml(tbModel.getTestCaseList(), tbModel.getPrecisionModel()) );
-      }
-    }
-    catch (Exception x) {
-      SwingUtil.reportException(this, x);
-    }
-  }
-
   public String getRunXml() 
   {
   	return XMLTestWriter.getRunXml(tbModel.getTestCaseList(), tbModel.getPrecisionModel());
   }
-  
-  void menuSaveAsJava_actionPerformed(ActionEvent e) {
-    try {
-      fileChooser.removeChoosableFileFilter(SwingUtil.XML_FILE_FILTER);
-      fileChooser.addChoosableFileFilter(SwingUtil.JAVA_FILE_FILTER);
-      fileChooser.setDialogTitle("Save Java File");
-      if (JFileChooser.APPROVE_OPTION == fileChooser.showSaveDialog(this)) {
-        File file = fileChooser.getSelectedFile();
-        if (! SwingUtil.confirmOverwrite(this, file)) return;
-        String className = fileChooser.getSelectedFile().getName();
-        int extensionIndex = className.lastIndexOf(".");
-        if (extensionIndex > 0) {
-          className = className.substring(0, extensionIndex);
-        }
-        ;
-        FileUtil.setContents(fileChooser.getSelectedFile().getPath(), JavaTestWriter.getRunJava(className, tbModel));
-      }
-    }
-    catch (Exception x) {
-      SwingUtil.reportException(this, x);
-    }
-  }
-
-  void menuSaveAsHtml_actionPerformed(ActionEvent e) {
-    try {
-      directoryChooser.setDialogTitle("Select Folder In Which To Save HTML and GIF Files");
-      if (JFileChooser.APPROVE_OPTION == directoryChooser.showSaveDialog(this)) {
-        int choice = JOptionPane.showConfirmDialog(this,
-            "Would you like the spatial function images "
-             + "to show the A and B geometries?", "Confirmation",
-            JOptionPane.YES_NO_CANCEL_OPTION);
-        final HtmlWriter writer = new HtmlWriter();
-        switch (choice) {
-          case JOptionPane.CANCEL_OPTION:
-            return;
-          case JOptionPane.YES_OPTION:
-            writer.setShowingABwithSpatialFunction(true);
-            break;
-          case JOptionPane.NO_OPTION:
-            writer.setShowingABwithSpatialFunction(false);
-            break;
-        }
-        final File directory = directoryChooser.getSelectedFile();
-        Assert.isTrue(directory.exists());
-        //        BusyDialog.setOwner(this);
-        //        BusyDialog busyDialog = new BusyDialog();
-        //        writer.setBusyDialog(busyDialog);
-        //        try {
-        //          busyDialog.execute("Saving .html and .gif files", new BusyDialog.Executable() {
-        //            public void execute() throws Exception {
-        writer.write(directory, tbModel.getTestCaseList(), tbModel.getPrecisionModel());
-        //            }
-        //          });
-        //        }
-        //        catch (Exception e2) {
-        //          System.out.println(busyDialog.getStackTrace());
-        //          throw e2;
-        //        }
-      }
-    }
-    catch (Exception x) {
-      SwingUtil.reportException(this, x);
-    }
-  }
-
-  public void actionSaveImageAsPNG() {
-    initFileChoosers();
-    try {
-      String fullFileName = SwingUtil.chooseFilenameWithConfirm(this, pngFileChooser);  
-      if (fullFileName == null) return;
-        ImageUtil.writeImage(testCasePanel.getGeometryEditPanel(), 
-            fullFileName,
-            ImageUtil.IMAGE_FORMAT_NAME_PNG);
-    }
-    catch (Exception x) {
-      reportException(x);
-    }
-  }
-
-  public void actionSaveImageToClipboard() {
-    try {
-        ImageUtil.saveImageToClipboard(testCasePanel.getGeometryEditPanel(), 
-        		ImageUtil.IMAGE_FORMAT_NAME_PNG);
-    }
-    catch (Exception x) {
-      reportException(x);
-    }
-  }
 
   void actionDeleteAllTestCases() {
     tbModel.cases().init();
@@ -561,28 +391,12 @@ public class JTSTestBuilderFrame extends JFrame
     }
   }
 
-  void precisionModelMenuItem_actionPerformed() {
-    try {
-      PrecisionModelDialog precisionModelDialog = new PrecisionModelDialog(
-          this, "Edit Precision Model", true);
-      GuiUtil.center(precisionModelDialog, this);
-      precisionModelDialog.setPrecisionModel(tbModel.getPrecisionModel());
-      precisionModelDialog.setVisible(true);
-      tbModel.changePrecisionModel(precisionModelDialog.getPrecisionModel());
-      updatePrecisionModelDescription();
-      updateGeometry();
-    }
-    catch (ParseException pe) {
-      reportException(pe);
-    }
-  }
-  void revealTopo_actionPerformed() {
+  private void displayRevealTopo() {
     DisplayParameters.setRevealingTopology(testCasePanel.cbRevealTopo.isSelected());
     DisplayParameters.setTopologyStretchSize(testCasePanel.getStretchSize());
     JTSTestBuilder.controller().geometryViewChanged();
   }
 
-
   /**
    *  Component initialization
    */
@@ -598,7 +412,7 @@ public class JTSTestBuilderFrame extends JFrame
     directoryChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
     directoryChooser.setMultiSelectionEnabled(false);
     //Center the window
-    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
+    //Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
     
     //---------------------------------------------------
     
@@ -620,14 +434,14 @@ public class JTSTestBuilderFrame extends JFrame
     //---- Input tabs
     inputTabbedPane.setTabPlacement(JTabbedPane.LEFT);
     inputTabbedPane.add(testListPanel, AppStrings.TAB_LABEL_CASES);
+    inputTabbedPane.add(layerListPanel, AppStrings.TAB_LABEL_LAYERS);
     inputTabbedPane.add(wktPanel,  AppStrings.TAB_LABEL_INPUT);
     inputTabbedPane.add(resultWKTPanel, AppStrings.TAB_LABEL_RESULT);
     inputTabbedPane.add(resultValuePanel, AppStrings.TAB_LABEL_VALUE);
-    inputTabbedPane.add(commandPanel,  AppStrings.TAB_LABEL_COMMAND);
     inputTabbedPane.add(inspectPanel,  AppStrings.TAB_LABEL_INSPECT);
     inputTabbedPane.add(statsPanel, AppStrings.TAB_LABEL_STATS);
     inputTabbedPane.add(logPanel, AppStrings.TAB_LABEL_LOG);
-    inputTabbedPane.add(layerListPanel, AppStrings.TAB_LABEL_LAYERS);
+    inputTabbedPane.add(commandPanel,  AppStrings.TAB_LABEL_COMMAND);
     inputTabbedPane.setSelectedIndex(1);
     inputTabbedPane.addChangeListener(new ChangeListener() {
       public void stateChanged(ChangeEvent e)
@@ -671,7 +485,7 @@ public class JTSTestBuilderFrame extends JFrame
     }   
   }
   
-  private void updateGeometry() {
+  public void geometryChanged() {
     testCasePanel.relatePanel.clearResults();
     testCasePanel.setTestCase(currentCase());
     updateWktPanel();
@@ -684,7 +498,7 @@ public class JTSTestBuilderFrame extends JFrame
     wktPanel.setText(g1, 1);
   }
 
-  private void updatePrecisionModelDescription() {
+  void updatePrecisionModelDescription() {
     testCasePanel.setPrecisionModelDescription(tbModel.getPrecisionModel().toString());
   }
 
@@ -714,19 +528,6 @@ public class JTSTestBuilderFrame extends JFrame
         "Error", JOptionPane.ERROR_MESSAGE);
   }
 
-  void menuRemoveDuplicatePoints_actionPerformed(ActionEvent e) {
-    CleanDuplicatePoints clean = new CleanDuplicatePoints();
-    Geometry cleanGeom = clean.clean(tbModel.getGeometryEditModel().getGeometry(0));
-    currentCase().setGeometry(0, cleanGeom);
-    updateGeometry();
-  }
-
-  void menuChangeToLines_actionPerformed(ActionEvent e) {
-    Geometry cleanGeom = LinearComponentExtracter.getGeometry(tbModel.getGeometryEditModel().getGeometry(0));
-    currentCase().setGeometry(0, cleanGeom);
-    updateGeometry();
-  }
-
   private Coordinate pickOffset(Geometry a, Geometry b) {
     if (a != null && ! a.isEmpty()) {
       return a.getCoordinates()[0];
