@@ -21,12 +21,12 @@ import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 
-import javax.swing.AbstractButton;
 import javax.swing.BorderFactory;
 import javax.swing.Box;
 import javax.swing.BoxLayout;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
+import javax.swing.JComboBox;
 import javax.swing.JComponent;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
@@ -44,10 +44,13 @@ import org.locationtech.jtstest.testbuilder.ui.ColorUtil;
 import org.locationtech.jtstest.testbuilder.ui.SwingUtil;
 import org.locationtech.jtstest.testbuilder.ui.style.BasicStyle;
 import org.locationtech.jtstest.testbuilder.ui.style.LayerStyle;
+import org.locationtech.jtstest.testbuilder.ui.style.Palette;
+import org.locationtech.jtstest.testbuilder.ui.style.VertexStyle;
 
 public class LayerStylePanel extends JPanel {
   private Layer layer;
   private JLabel title;
+  private JCheckBox cbShift;
   private JPanel stylePanel;
   private int rowIndex;
   private JCheckBox cbDashed;
@@ -61,12 +64,12 @@ public class LayerStylePanel extends JPanel {
   
   private JCheckBox cbVertex;
   private JPanel btnVertexColor;
-  private JSpinner spinnerVertexSize;
+  private JSpinner spinVertexSize;
   private SpinnerNumberModel vertexSizeModel;
   
   private JCheckBox cbLabel;
   private JPanel btnLabelColor;
-  private JSpinner spinnerLabelSize;
+  private JSpinner spinLabelSize;
   private SpinnerNumberModel labelSizeModel;
   
   private JCheckBox cbStroked;
@@ -75,9 +78,12 @@ public class LayerStylePanel extends JPanel {
   private JCheckBox cbStructure;
   private JCheckBox cbVertexLabel;
   private JCheckBox cbOffset;
-  private JSpinner spinnerOffsetSize;
+  private JSpinner spinOffsetSize;
   private SpinnerNumberModel offsetSizeModel;
   private JCheckBox cbEndpoint;
+  private JComboBox comboPalette;
+  private JCheckBox cbSegIndex;
+  private JComboBox comboVertexSymbol;
 
   
   public LayerStylePanel() {
@@ -98,20 +104,24 @@ public class LayerStylePanel extends JPanel {
     txtName.setEditable(isModifiable);
     txtName.setFocusable(isModifiable);
 
+    cbShift.setSelected(layer.getLayerStyle().isShifted());
     cbVertex.setSelected(layer.getLayerStyle().isVertices());
     cbVertexLabel.setSelected(layer.getLayerStyle().isVertexLabels());
+    setVertexSymbol(comboVertexSymbol, layer.getLayerStyle().getVertexSymbol());
     vertexSizeModel.setValue(layer.getLayerStyle().getVertexSize());
     cbLabel.setSelected(layer.getLayerStyle().isLabel());
     labelSizeModel.setValue(layer.getLayerStyle().getLabelSize());
     cbEndpoint.setSelected(layer.getLayerStyle().isEndpoints());
     cbDashed.setSelected(geomStyle().isDashed());
     cbOffset.setSelected(layer.getLayerStyle().isOffset());
-    offsetSizeModel.setValue( layer.getLayerStyle().getOffsetSize() );
+    offsetSizeModel.setValue(layer.getLayerStyle().getOffsetSize() );
     cbStroked.setSelected(geomStyle().isStroked());
     cbFilled.setSelected(geomStyle().isFilled());
     cbOrient.setSelected(layer.getLayerStyle().isOrientations());
-    cbStructure.setSelected(layer.getLayerStyle().isOrientations());
-    lineWidthModel.setValue(geomStyle().getStrokeWidth());
+    cbStructure.setSelected(layer.getLayerStyle().isStructure());
+    cbSegIndex.setSelected(layer.getLayerStyle().isSegIndex());
+    lineWidthModel.setValue((double) geomStyle().getStrokeWidth());
+    setPaletteType(comboPalette, layer.getLayerStyle().getFillType());
     updateStyleControls();
   }
   
@@ -172,7 +182,20 @@ public class LayerStylePanel extends JPanel {
     txtName.setMaximumSize(new Dimension(100,20));
     txtName.setPreferredSize(new Dimension(100,20));
     txtName.setMinimumSize(new Dimension(100,20));
-    addRow("Name", txtName);
+    
+    cbShift = new JCheckBox();
+    cbShift.setToolTipText(AppStrings.TIP_STYLE_SHIFT);
+    cbShift.setAlignmentX(Component.LEFT_ALIGNMENT);
+    cbShift.setText("Shift");
+    cbShift.addActionListener(new java.awt.event.ActionListener() {
+      public void actionPerformed(ActionEvent e) {
+        if (layer == null) return;
+        layer.getLayerStyle().setShift(cbShift.isSelected());
+        JTSTestBuilder.controller().geometryViewChanged();
+      }
+    });
+
+    addRow("Name", txtName, cbShift);
     
     txtName.getDocument().addDocumentListener(new DocumentListener() {
       public void changedUpdate(DocumentEvent e) {
@@ -216,11 +239,11 @@ public class LayerStylePanel extends JPanel {
         }
        );
     
-    vertexSizeModel = new SpinnerNumberModel(4.0, 0, 100.0, 1);
-    spinnerVertexSize = new JSpinner(vertexSizeModel);
-    spinnerVertexSize.setMaximumSize(new Dimension(40,16));
-    spinnerVertexSize.setAlignmentX(Component.LEFT_ALIGNMENT);
-    spinnerVertexSize.addChangeListener(new ChangeListener() {
+    vertexSizeModel = new SpinnerNumberModel(4, 0, 100, 1);
+    spinVertexSize = new JSpinner(vertexSizeModel);
+    spinVertexSize.setMaximumSize(new Dimension(40,16));
+    spinVertexSize.setAlignmentX(Component.LEFT_ALIGNMENT);
+    spinVertexSize.addChangeListener(new ChangeListener() {
       public void stateChanged(ChangeEvent e) {
         int size = vertexSizeModel.getNumber().intValue();
         layer.getLayerStyle().setVertexSize(size);
@@ -239,8 +262,19 @@ public class LayerStylePanel extends JPanel {
       }
     });
 
+    comboVertexSymbol = new JComboBox(vertexSymbolNames);
+    comboVertexSymbol.addActionListener(new ActionListener() {
+      public void actionPerformed(ActionEvent e) {
+        JComboBox cb = (JComboBox)e.getSource();
+        int symType = getVertexSymbol(cb);
+        layer.getLayerStyle().setVertexSymbol(symType);
+        JTSTestBuilder.controller().geometryViewChanged();
+      }
+    });
     
-    addRow("Vertices", cbVertex, btnVertexColor, spinnerVertexSize, cbVertexLabel);
+    comboVertexSymbol.setToolTipText(AppStrings.TIP_STYLE_SYMBOL);
+   
+    addRow("Vertices", cbVertex, btnVertexColor, spinVertexSize, comboVertexSymbol, cbVertexLabel);
     //=============================================
 
     cbStroked = new JCheckBox();
@@ -265,7 +299,7 @@ public class LayerStylePanel extends JPanel {
           }
         }
        );
-    JButton btnVertexSynch = SwingUtil.createButton("^", "Synch Vertex Color", new ActionListener() {
+    JButton btnVertexSynch = createSynchButton("^", "Synch Vertex Color", new ActionListener() {
       public void actionPerformed(ActionEvent arg0) {
         if (layer == null) return;
         Color clr = ColorControl.getColor(btnLineColor);
@@ -276,7 +310,7 @@ public class LayerStylePanel extends JPanel {
       }
     });
 
-    lineWidthModel = new SpinnerNumberModel(1.0, 0, 100.0, 0.2);
+    lineWidthModel = new SpinnerNumberModel(1.0, 0, 100, 0.2);
     spinnerLineWidth = new JSpinner(lineWidthModel);
     //widthSpinner.setMinimumSize(new Dimension(50,12));
     //widthSpinner.setPreferredSize(new Dimension(50,12));
@@ -303,9 +337,6 @@ public class LayerStylePanel extends JPanel {
         }
       }
     });
-    addRow("Line", cbStroked, btnLineColor, btnVertexSynch, spinnerLineWidth, sliderLineAlpha);
-
-    //=============================================
     cbDashed = new JCheckBox();
     cbDashed.setText("Dashed");
     //cbDashed.setToolTipText(AppStrings.STYLE_VERTEX_ENABLE);
@@ -317,6 +348,33 @@ public class LayerStylePanel extends JPanel {
         JTSTestBuilder.controller().geometryViewChanged();
       }
     });
+    cbOffset = new JCheckBox();
+    cbOffset.setText("Offset");
+    //cbDashed.setToolTipText(AppStrings.STYLE_VERTEX_ENABLE);
+    cbOffset.setAlignmentX(Component.LEFT_ALIGNMENT);
+    cbOffset.addActionListener(new java.awt.event.ActionListener() {
+      public void actionPerformed(ActionEvent e) {
+        if (layer == null) return;
+        layer.getLayerStyle().setOffset(cbOffset.isSelected());
+        JTSTestBuilder.controller().geometryViewChanged();
+      }
+    });
+    offsetSizeModel = new SpinnerNumberModel(LayerStyle.INIT_OFFSET_SIZE, -100, 100, 1);
+    spinOffsetSize = new JSpinner(offsetSizeModel);
+    spinOffsetSize.setMaximumSize(new Dimension(40,16));
+    spinOffsetSize.setAlignmentX(Component.LEFT_ALIGNMENT);
+    spinOffsetSize.addChangeListener(new ChangeListener() {
+      public void stateChanged(ChangeEvent e) {
+        int size = offsetSizeModel.getNumber().intValue();
+        layer.getLayerStyle().setOffsetSize(size);
+        JTSTestBuilder.controller().geometryViewChanged();
+      }
+    });
+
+    addRow("Line", cbStroked, btnLineColor, btnVertexSynch, sliderLineAlpha, spinnerLineWidth, 
+        cbDashed, cbOffset, spinOffsetSize);
+
+    //=============================================
     
     cbEndpoint = new JCheckBox();
     cbEndpoint.setText("Endpoints");
@@ -351,31 +409,20 @@ public class LayerStylePanel extends JPanel {
       }
     });
 
-    cbOffset = new JCheckBox();
-    cbOffset.setText("Offset");
-    //cbDashed.setToolTipText(AppStrings.STYLE_VERTEX_ENABLE);
-    cbOffset.setAlignmentX(Component.LEFT_ALIGNMENT);
-    cbOffset.addActionListener(new java.awt.event.ActionListener() {
+    cbSegIndex = new JCheckBox();
+    cbSegIndex.setText("Index");
+    cbSegIndex.setAlignmentX(Component.LEFT_ALIGNMENT);
+    cbSegIndex.addActionListener(new java.awt.event.ActionListener() {
       public void actionPerformed(ActionEvent e) {
         if (layer == null) return;
-        layer.getLayerStyle().setOffset(cbOffset.isSelected());
-        JTSTestBuilder.controller().geometryViewChanged();
-      }
-    });
-    offsetSizeModel = new SpinnerNumberModel(LayerStyle.INIT_OFFSET_SIZE, -100, 100.0, 1);
-    spinnerOffsetSize = new JSpinner(offsetSizeModel);
-    spinnerOffsetSize.setMaximumSize(new Dimension(40,16));
-    spinnerOffsetSize.setAlignmentX(Component.LEFT_ALIGNMENT);
-    spinnerOffsetSize.addChangeListener(new ChangeListener() {
-      public void stateChanged(ChangeEvent e) {
-        int size = offsetSizeModel.getNumber().intValue();
-        layer.getLayerStyle().setOffsetSize(size);
+        layer.getLayerStyle().setSegIndex(cbSegIndex.isSelected());
         JTSTestBuilder.controller().geometryViewChanged();
       }
     });
+
     
    // Leave on separate line to allow room for dash style
-    addRow("", cbDashed, cbEndpoint, cbOrient, cbStructure, cbOffset, spinnerOffsetSize);
+    addRow("", cbEndpoint, cbOrient, cbStructure, cbSegIndex);
     //=============================================
 
     cbFilled = new JCheckBox();
@@ -412,7 +459,7 @@ public class LayerStylePanel extends JPanel {
           }
         }
        );
-    JButton btnLineSynch = SwingUtil.createButton("^", "Synch Line Color", new ActionListener() {
+    JButton btnLineSynch = createSynchButton("^", "Synch Line Color", new ActionListener() {
       public void actionPerformed(ActionEvent arg0) {
         Color clr = lineColorFromFill( ColorControl.getColor(btnFillColor));
         geomStyle().setLineColor(clr );
@@ -425,6 +472,21 @@ public class LayerStylePanel extends JPanel {
 
     //=============================================
 
+    comboPalette = new JComboBox(paletteNames);
+    comboPalette.addActionListener(new ActionListener() {
+      public void actionPerformed(ActionEvent e) {
+        JComboBox cb = (JComboBox)e.getSource();
+        int fillType = getPaletteType(cb);
+        layer.getLayerStyle().setFillType(fillType);
+        JTSTestBuilder.controller().geometryViewChanged();
+      }
+    });
+    comboPalette.setToolTipText(AppStrings.TIP_STYLE_PALETTE);
+    addRow("Palette", comboPalette);
+    
+    //=============================================
+
+    
     cbLabel = new JCheckBox();
     //cbLabel.setToolTipText(AppStrings.TIP_STYLE_VERTEX_ENABLE);
     cbLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
@@ -447,11 +509,11 @@ public class LayerStylePanel extends JPanel {
         }
        );
     
-    labelSizeModel = new SpinnerNumberModel(4.0, 0, 100.0, 1);
-    spinnerLabelSize = new JSpinner(labelSizeModel);
-    spinnerLabelSize.setMaximumSize(new Dimension(40,16));
-    spinnerLabelSize.setAlignmentX(Component.LEFT_ALIGNMENT);
-    spinnerLabelSize.addChangeListener(new ChangeListener() {
+    labelSizeModel = new SpinnerNumberModel(4, 0, 100, 1);
+    spinLabelSize = new JSpinner(labelSizeModel);
+    spinLabelSize.setMaximumSize(new Dimension(40,16));
+    spinLabelSize.setAlignmentX(Component.LEFT_ALIGNMENT);
+    spinLabelSize.addChangeListener(new ChangeListener() {
       public void stateChanged(ChangeEvent e) {
         int size = labelSizeModel.getNumber().intValue();
         layer.getLayerStyle().setLabelSize(size);
@@ -461,20 +523,71 @@ public class LayerStylePanel extends JPanel {
 
 
     
-    addRow("Label", cbLabel, btnLabelColor, spinnerLabelSize);
+    addRow("Label", cbLabel, btnLabelColor, spinLabelSize);
     
     //=============================================
     
     return containerPanel;
   }
 
+  //-----------------------------------------
+  static String[] paletteNames = { "Basic", "Varying", "Spectrum", "Spectrum Random" }; 
+
+  private static int getPaletteType(JComboBox comboPal) {
+    String palName = (String)comboPal.getSelectedItem();
+    
+    int paletteType = Palette.TYPE_BASIC;
+    if (palName.equalsIgnoreCase(paletteNames[1])) paletteType = Palette.TYPE_VARY;
+    if (palName.equalsIgnoreCase(paletteNames[2])) paletteType = Palette.TYPE_SPECTRUM;
+    if (palName.equalsIgnoreCase(paletteNames[3])) paletteType = Palette.TYPE_SPECTRUM_RANDOM;
+    return paletteType;
+  }
+  
+  private static void setPaletteType(JComboBox comboPal, int paletteType) {
+    int index = 0;
+    if (paletteType == Palette.TYPE_VARY) index = 1;
+    if (paletteType == Palette.TYPE_SPECTRUM) index = 2;
+    if (paletteType == Palette.TYPE_SPECTRUM_RANDOM) index = 3;
+    comboPal.setSelectedIndex(index);
+  }
+  
+  //-----------------------------------------
+  static String[] vertexSymbolNames = { "Square", "Square Hollow", "Circle", "Circle Hollow" }; 
+
+  private static int getVertexSymbol(JComboBox combo) {
+    String name = (String)combo.getSelectedItem();
+    
+    for (int i = 0; i < vertexSymbolNames.length; i++) {
+      if (name.equalsIgnoreCase(vertexSymbolNames[i])) return i;
+    }
+    return VertexStyle.SYM_SQUARE_SOLID;
+  }
+  
+  private static void setVertexSymbol(JComboBox combo, int symbolType) {
+    combo.setSelectedIndex(symbolType);
+  }
+  
   protected static Color lineColorFromFill(Color clr) {
     return ColorUtil.saturate(clr,  1);
     //return clr.darker();
   }
 
+  private JButton createSynchButton(String lbl, String tip, ActionListener actionListener) {
+    JButton btn = SwingUtil.createButton(lbl, tip, actionListener);
+    btn.setMargin(new Insets(0, 0, 0, 0));
+    Dimension dim = new Dimension(16, 20);
+    btn.setMinimumSize(dim);
+    btn.setPreferredSize(dim);
+    btn.setMaximumSize(dim);
+    return btn;
+  }
+  
   private JSlider createOpacitySlider(ChangeListener changeListener) {
     JSlider slide = new JSlider(JSlider.HORIZONTAL, 0, 255, 150);
+    Dimension dim = new Dimension(80, 20);
+    slide.setMinimumSize(dim);
+    slide.setPreferredSize(dim);
+    slide.setMaximumSize(dim);
     slide.addChangeListener(changeListener);
     slide.setMajorTickSpacing(32);
     slide.setPaintTicks(true);
