@@ -1,5 +1,3 @@
-
-
 /*
  * Copyright (c) 2016 Vivid Solutions.
  *
@@ -168,7 +166,7 @@ public class LineSegment
   
   /**
    * Determines the orientation index of a {@link Coordinate} relative to this segment.
-   * The orientation index is as defined in {@link Orientation#computeOrientation}.
+   * The orientation index is as defined in {@link Orientation#index(Coordinate, Coordinate, Coordinate)}.
    *
    * @param p the coordinate to compare
    *
@@ -176,7 +174,7 @@ public class LineSegment
    * @return -1 (RIGHT) if <code>p</code> is to the right of this segment
    * @return 0 (COLLINEAR) if <code>p</code> is collinear with this segment
    * 
-   * @see Orientation#computeOrientation(Coordinate, Coordinate, Coordinate)
+   * @see Orientation#index(Coordinate, Coordinate, Coordinate)
    */
   public int orientationIndex(Coordinate p)
   {
@@ -282,7 +280,7 @@ public class LineSegment
    */
   public Coordinate pointAlong(double segmentLengthFraction)
   {
-    Coordinate coord = new Coordinate();
+    Coordinate coord = p0.create();
     coord.x = p0.x + segmentLengthFraction * (p1.x - p0.x);
     coord.y = p0.y + segmentLengthFraction * (p1.y - p0.y);
     return coord;
@@ -328,7 +326,9 @@ public class LineSegment
     double offsetx = segx - uy;
     double offsety = segy + ux;
 
-    Coordinate coord = new Coordinate(offsetx, offsety);
+    Coordinate coord = p0.create();
+    coord.setX(offsetx);
+    coord.setY(offsety);
     return coord;
   }
 
@@ -406,10 +406,10 @@ public class LineSegment
    */
   public Coordinate project(Coordinate p)
   {
-    if (p.equals(p0) || p.equals(p1)) return new Coordinate(p);
+    if (p.equals(p0) || p.equals(p1)) return p.copy();
 
     double r = projectionFactor(p);
-    Coordinate coord = new Coordinate();
+    Coordinate coord = p.copy();
     coord.x = p0.x + r * (p1.x - p0.x);
     coord.y = p0.y + r * (p1.y - p0.y);
     return coord;
@@ -445,6 +445,24 @@ public class LineSegment
     return new LineSegment(newp0, newp1);
   }
   
+  /**
+   * Computes the {@link LineSegment} that is offset from 
+   * the segment by a given distance.
+   * The computed segment is offset to the left of the line if the offset distance is
+   * positive, to the right if negative.
+   *
+   * @param offsetDistance the distance the point is offset from the segment
+   *    (positive is to the left, negative is to the right)
+   * @return a line segment offset by the specified distance
+   * 
+   * @throws IllegalStateException if the segment has zero length
+   */
+  public LineSegment offset(double offsetDistance) {
+    Coordinate offset0 = pointAlongOffset(0, offsetDistance);
+    Coordinate offset1 = pointAlongOffset(1, offsetDistance);
+    return new LineSegment(offset0, offset1);
+  }
+  
   /**
    * Computes the reflection of a point in the line defined
    * by this line segment.
@@ -467,7 +485,10 @@ public class LineSegment
     double rx = ( -A2subB2*x - 2*A*B*y - 2*A*C ) / A2plusB2;
     double ry = ( A2subB2*y - 2*A*B*x - 2*B*C ) / A2plusB2;
     
-    return new Coordinate(rx, ry);
+    Coordinate coord = p.copy();
+    coord.setX(rx);
+    coord.setY(ry);
+    return coord;
   }
   
   /**
@@ -620,6 +641,15 @@ public class LineSegment
    * @return a hashcode for this object
    */
   public int hashCode() {
+    int hash = 17;
+    hash = hash * 29 + Double.hashCode(p0.x);
+    hash = hash * 29 + Double.hashCode(p0.y);
+    hash = hash * 29 + Double.hashCode(p1.x);
+    hash = hash * 29 + Double.hashCode(p1.y);
+    return hash;
+  }
+
+  public int OLDhashCode() {
     long bits0 = java.lang.Double.doubleToLongBits(p0.x);
     bits0 ^= java.lang.Double.doubleToLongBits(p0.y) * 31;
     int hash0 = (((int) bits0) ^ ((int) (bits0  >> 32)));
@@ -631,7 +661,7 @@ public class LineSegment
     // XOR is supposed to be a good way to combine hashcodes
     return hash0 ^ hash1;
   }
-
+  
   /**
    *  Compares this object with the specified object for order.
    *  Uses the standard lexicographic ordering for the points in the LineSegment.
