@@ -935,7 +935,7 @@ public class QuadEdgeSubdivision {
     coordList.closeRing();
     
     if (coordList.size() < 4) {
-      System.out.println(coordList);
+      //System.out.println(coordList);
       coordList.add(coordList.get(coordList.size()-1), true);
     }
     
