@@ -12,6 +12,7 @@
 
 package org.locationtech.jts.geom.impl;
 
+import org.locationtech.jts.geom.CoordinateSequence;
 import org.locationtech.jts.geom.CoordinateSequenceFactory;
 
 import junit.textui.TestRunner;
@@ -38,4 +39,17 @@ public class PackedCoordinateSequenceDoubleTest
     return PackedCoordinateSequenceFactory.DOUBLE_FACTORY;
   }
 
+  public void test3dCoordinateSequence() {
+    CoordinateSequence cs = new PackedCoordinateSequenceFactory(PackedCoordinateSequenceFactory.DOUBLE)
+            .create(new double[]{0.0,1.0,2.0,3.0,4.0,5.0}, 3);
+    assertEquals(2.0, cs.getCoordinate(0).getZ());
+  }
+
+  public void test4dCoordinateSequence() {
+    CoordinateSequence cs = new PackedCoordinateSequenceFactory(PackedCoordinateSequenceFactory.DOUBLE)
+            .create(new double[]{0.0,1.0,2.0,3.0,4.0,5.0,6.0,7.0}, 4);
+    assertEquals(2.0, cs.getCoordinate(0).getZ());
+    assertEquals(3.0, cs.getCoordinate(0).getM());
+  }
+
 }
